exports.sections =
  
  'Overview': -> 
    $ """
      *Edit the CoffeeScript on the left. Markup updates on the right.*
      
      Tags are JavaScript functions that produce markup elements.
      """
    
    example cs: "a()"

    $ "Tags take arguments. String and number arguments become text nodes..."
    
    example cs: "a 'hi'"
    
    
    $ "...object arguments become attributes..."
    
    example cs: "a foo:3, bar:'envy'"
    
    
    $ "...node arguments become children..."
    
    example cs: "a b c"
    
    $ """
      ...array arguments are flattened and each array element is processed
      as an argument. 
      """
    
    example cs: "a (b str for str in ['x', 'y', 'z'])"     
    
    
    $ "Tags can be called with an arbitrary number of arguments."
    
    example cs: """
      a
        foo:'/flake'
        b 'jot'
        c 'tackle'
          b {x} for x in [1..5]
      """
    
  
  
  'Text': ->
    
    $ "All text is escaped by default"
    example cs: " a '<&>'"
    
    $ "To avoid escaping text, prefix the text with a backtick "
    example cs: " a '`<&>'"
    
    
    $ """
      For convenience in an HTML context, text that starts with a period '.' or 
      a hash mark '#' are treated as classes and 
      id attributes, respectively, if the rest of the string constitutes a valid class or id. 
      """
      
    example cs: "a '.navlink', '#toc', 'Table of contents'"
    
    
    $ "Markdown can be used anywhere, using the `$` function."
    
    example cs: """
      $ ""\"
        Shopping list
        -------------
        - milk
        - eggs
        - nunchucks
        - vegetables
          - carrots
          - brocolli
        - *soup*
        ""\"
        """
    
    
  'Mixins': ->
    $ """
      A function passed as an argument to a tag gets *mixed-in* to the resulting element.
      Some functions in scope like `attr`, `text` and `call` allow you to manipulate the
      current element from within the mixin. 
      
      Any top-level elements created in the mixin get added to the current tag.
      
      """
      
    example cs: """
        a ->
          
          
      """

    $ "Anonymous mixins are useful for doing arbitray local computations"

  'Macros': ->
  
    example cs: """
      data =
        'Get it': '#download'
        'Flaunt it': '#showoff'
      
      item = (label, href) -> li a {href}, label 
      
      ol (item key, val for key, val of data)
      """
    
  'Templates': ->
    $ "A template is a string with inline code. They are usually encapsulated in a function."
    
  
    
  
    $ """
      define your own jotlets 

      """
      
    codeblock """
      javascript: -> 
        script {type:'text/javascript'}, arguments...
      """
      
    example cs: """
      nav = -> _ '.nav', arguments...
      
      nav '.gonuts', 'Go Nuts',
        img src: 'images/gonuts.png'
      """
    
    example cs: """
        $ ""\"
          Shopping list
          -------------
          - milk
          - eggs
          - nunchucks
          - vegetables
            - carrots
            - brocolli
          - *soup*
          ""\"
          """
          
    $ """
      This is a quick example:
      """
      
    
      
    $ """ 
      There's nothing really to say about that.
      """
      
    example 
      description: 'Overview'
      cs: """
        a()
        
        
        a b c 'gravy'
        
        
        
        a foo: 3, bar: 'super'
        
        # underscore '_' is short for 'div'
        _ '#footer'
        
        # Javascript tags
        javascript cdata ""\"
          (function() {
            var foo = 3
            return function(x){ return foo < 3 }
          })()
          ""\"
        
        note 'You can use markdown!'
        $ ""\"
          Party Mix
          ---------
          - Coldplay
            - Yellow
            - Fix you
          - Tori Amos
            - Winter
            - Jupiter
          ""\"
      """
    
    $ """
      This is more information.
      """
    