html = null

setup = ->
  html = new Jot.HtmlContext
  html.declareTag 'c'
  html

testjot = (description, locals, fn, expected) ->  
  setup()
  test description, ->
    check locals, fn, expected

check = (locals, fn, expected) ->
  if not (fn? and fn.call and fn.apply)
    [locals, fn, expected] = [null, arguments...]
  r = html.run locals, fn
  if r isnt expected 
    console.log 'Expected:\n', expected, 'Actual:\n', r
  eq r, expected
    
  

test 'scopjs', ->

testjot 'self-closing tag', ->
  a()
, """
  <a/>
  """ 


testjot 'text mixin', -> 
  a 'asdf'
, """
  <a>asdf</a>
  """

testjot 'node as mixin', ->
  a b()
, """
  <a>
    <b/>
  </a>
  """

testjot 'nested node mixins', ->
  a 'foo',
    b 'rock ', 
      c 'star'
      
, """
  <a>foo
    <b>rock 
      <c>star</c>
    </b>
  </a>
  """


testjot 'attr hash mixin', ->
  a 
    foo: 3
    bar: 'great'
, """
  <a foo="3" bar="great"/>
  """


testjot 'Array mixin', ->
  a [
    'blue'
    b 'Cheese'
  ]
, """
  <a>blue
    <b>Cheese</b>
  </a>
  """

testjot 'Array of array mixin', ->
  a [[ c 'old'],
    'blue'
    b 'Cheese'
  ]
, """
  <a>
    <c>old</c>blue
    <b>Cheese</b>
  </a>
  """

testjot 'Function mixin', ->
  a ->
    b 'great'
    br()
, """
  <a>
    <b>great</b>
    <br/>
  </a>
  """

testjot 'Nested function mixin', ->
  h1 ->
    p 'good'
    a ->
      b 'great'
      br()
, """
  <h1>
    <p>good</p>
    <a>
      <b>great</b>
      <br/>
    </a>
  </h1>
  """


# local node manipulation
testjot 'addClass in mixin fn', ->
  h1 
    class: 'foo'
    -> addClass 'super'
, """
  <h1 class="foo super"/>
  """
  
test 'append in mixin fn', ->

testjot 'text in mixin fn', ->
  a -> text 'asdf'
, """
  <a>asdf</a>
  """

testjot 'attr in mixin fn', ->
  a -> attr foo:'3'
, """
  <a foo="3"/>
  """



testjot 'locals'
  foo: 3
  sub: (s)->s.substr 0,2
  -> a {foo}, sub 'hippo'
, """
  <a foo="3">hi</a>
  """
  
testjot 'nested functions with locals'
  foo: 3
  sub: (s)->s.substr 0,2
  -> 
    bar = 23
    a {foo}, ->
      b {bar}, sub 'hippo'
, """
  <a foo="3">
    <b bar="23">hi</b>
  </a>
  """

  
test 'this', ->
  # 'this' is the first argument to a function
  r = html.run ->
    a @text
  , text: 'asdf'
  eq r, "<a>asdf</a>"

test 'nested this', ->
  html.macro
    m2: ->
      b 'grey', ->
        a @foo
  
  check ->
    m2 foo:'asdf'
  , """
    <b>grey
      <a>asdf</a>
    </b>
    """
 


test 'macro', ->
  html.macro
    m1: (p1) ->
      b 'asdf'
      c p1
    
  check ->
    m1 'macro'
  , """
    <b>asdf</b>
    <c>macro</c>
    """

test 'macro with self argument', ->
  html.macro
    m2: ->
      b 'grey',
        a @foo
        
  check ->
    m2 foo:'asdf'
  , """
    <b>grey
      <a>asdf</a>
    </b>
    """

test 'mixin named function', ->
  html.scope
    mix1: -> 
      attr foo: 3
      b 'asdf'
      c 'cool'
      
  check ->
    a mix1
  , """
    <a foo="3">
      <b>asdf</b>
      <c>cool</c>
    </a>
    """

test "mixin called in macro with 'this'", ->
      
  html.scope
    mix2: ->
      attr {@foo}
      b {@bar}
  
  html.macro
    m4: ->
      a @bar, mix2

  check ->
    m4 foo: 3, bar: 'gravy'
  , """
    <a foo="3">gravy
      <b bar="gravy"/>
    </a>
    """
      

test 'macro called in macro', ->
  html.macro
    m1: (p1) ->
      b 'asdf'
      c p1
    m2: ->
      m1 'guu'
    
  check ->
    a m2()
  , """
    <a>
      <b>asdf</b>
      <c>guu</c>
    </a>
    """

test 'anonymous mixin as macro argument', ->
  html.macro
    m1: (p1) ->
      b 'asdf'
      c p1
      
  check ->
    m1 ->
      attr foo: '3'
      a 'go'
      b 'nested'
  , """
    <b>asdf</b>
    <c foo="3">
      <a>go</a>
      <b>nested</b>
    </c>
    """

test 'anonymous bound mixin as macro argument', ->
  html.macro
    m1: (p1) ->
      b 'asdf'
      c p1
      
  r = html.run ->
    m1 =>
      attr foo: @foo
      a 'go'
      b 'nested'
  , foo: 3
  eq r, 
    """
    <b>asdf</b>
    <c foo="3">
      <a>go</a>
      <b>nested</b>
    </c>
    """

test 'Mixin to a macro', ->
  html.scope
    mix2: (p1) ->
      ->
        attr bleh: p1
        c blu: 'gruv'
  
  html.macro
    m4: ->
      a @bar
      b @foo

  check ->
    _n = m4 bar: 'lazy', foo: 'rick'
    _n.call ' sloth', mix2 'good'
  , """
    <a bleh="good">lazy sloth
      <c blu="gruv"/>
    </a>
    <b bleh="good">rick sloth
      <c blu="gruv"/>
    </b>
    """ 
    
test 'Scoped MacroResult.call', ->
  html.macro macros =
    m1: (p1) ->
      a b c p1
    m2: (p1) ->
      a()
      b()
      c p1 
  prearg = (arg, macroargs...) ->
    nodes = macro.apply null, macroargs
    nodes.call arg
    
  for k, v of macros
    macro = html.local k
    name = "_#{k}"
    html.scope name, {macro}, prearg
    html.local(name).macro = macro
    
  check ->
    _m1 '.foo', 'envy'
    _m2 '#bar', 'cold'
  , """
    <a class="foo">
      <b>
        <c>envy</c>
      </b>
    </a>
    <a id="bar"/>
    <b id="bar"/>
    <c id="bar">cold</c>
    """


test 'external tag', ->
  r = html.a 'rock', -> b 'on'
  eq r,
    """
    <a>rock
      <b>on</b>
    </a>
    """

test 'external macro', ->
  html.macro
    m1: (p1) ->
      b 'asdf'
      c p1
  r = html.m1 'perky'
  eq r,
    """
    <b>asdf</b>
    <c>perky</c>
    """
    
test "externals dont leak orphans", ->
  html.a -> b 'q'
  eq html.state.orphans.length, 0
  html.run -> a -> b 'q'
  eq html.state.orphans.length, 0
  

  
testjot 'html class string mixin', ->
  a '.foo.bar'
, """
  <a class="foo bar"/>
  """

testjot 'html id string mixin', ->
  a '#foo'
, """
  <a id="foo"/>
  """

test 'toString method of single node', ->
  r = "" + html.local('a') 'perky'
  eq r, 
    """
    <a>perky</a>
    """
    
test 'toString method of macro result', ->
  html.macro
    m1: (p1) ->
      b 'asdf'
      c p1
  r = "" + html.local('m1') 'perky'
  eq r, 
    """
    <b>asdf</b>
    <c>perky</c>
    """
    
test 'template for string', ->
  r = html.run {foo:3}, ->
    p ->
      """
      `Lorem ipsum #{foo} dolor sit <b>amet</b>. #{em foo}
      Praesent rutrum #{b 'faucibus'}. That's what I 
      tell people
      """
  eq r,
      """
      <p>Lorem ipsum 3 dolor sit <b>amet</b>. <em>3</em>
        Praesent rutrum <b>faucibus</b>. That's what I 
        tell people</p>
      """
        
 
test 'markdown', ->
  r = html.markdown """
    Using this tool
    ---------------
    """
  eq r, """
    
    <h2>Using this tool</h2>
    """

test 'js functions as strings', ->
  r = html.run """
    function(){ a() }
    """
  eq r, "<a/>"

  r = html.run """
    (function(){ a() });
    """
  eq r, "<a/>"



    